@@ -1,12 +1,6 @@
 /***************************************************************************
-                          mergeresultwindow.h  -  description
-                             -------------------
-    begin                : Mon Mar 18 2002
-    copyright            : (C) 2002-2007 by Joachim Eibl
-    email                : joachim.eibl at gmx.de
- ***************************************************************************/
-
-/***************************************************************************
+ *   Copyright (C) 2002-2007 by Joachim Eibl <joachim.eibl at gmx.de>      *
+ *   Copyright (C) 2018 Michael Reeves reeves.87@gmail.com                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -19,106 +13,40 @@
 #define MERGERESULTWINDOW_H
 
 #include "diff.h"
+#include "selection.h"
+#include "Overview.h"
+#include "FileNameLineEdit.h"
 
 #include <QWidget>
-#include <QPixmap>
 #include <QTimer>
 #include <QStatusBar>
 #include <QTextLayout>
+#include <QLineEdit>
 
 class QPainter;
-
-class Overview : public QWidget
-{
-   Q_OBJECT
-public:
-   Overview( Options* pOptions );
-
-   void init( Diff3LineList* pDiff3LineList, bool bTripleDiff );
-   void reset();
-   void setRange( int firstLine, int pageHeight );
-   void setPaintingAllowed( bool bAllowPainting );
-
-   enum e_OverviewMode { eOMNormal, eOMAvsB, eOMAvsC, eOMBvsC };
-   void setOverviewMode( e_OverviewMode eOverviewMode );
-   e_OverviewMode getOverviewMode();
-
-public slots:
-   void setFirstLine(int firstLine);
-   void slotRedraw();
-signals:
-   void setLine(int);
-private:
-   const Diff3LineList* m_pDiff3LineList;
-   Options* m_pOptions;
-   bool m_bTripleDiff;
-   int m_firstLine;
-   int m_pageHeight;
-   QPixmap m_pixmap;
-   bool m_bPaintingAllowed;
-   e_OverviewMode m_eOverviewMode;
-   int m_nofLines;
-
-   virtual void paintEvent( QPaintEvent* e );
-   virtual void mousePressEvent( QMouseEvent* e );
-   virtual void mouseMoveEvent( QMouseEvent* e );
-   void drawColumn( QPainter& p, e_OverviewMode eOverviewMode, int x, int w, int h, int nofLines );
-};
-
-
-enum e_MergeDetails
-{
-   eDefault,
-   eNoChange,
-   eBChanged,
-   eCChanged,
-   eBCChanged,         // conflict
-   eBCChangedAndEqual, // possible conflict
-   eBDeleted,
-   eCDeleted,
-   eBCDeleted,         // possible conflict
-
-   eBChanged_CDeleted, // conflict
-   eCChanged_BDeleted, // conflict
-   eBAdded,
-   eCAdded,
-   eBCAdded,           // conflict
-   eBCAddedAndEqual    // possible conflict
-};
-
-void mergeOneLine( const Diff3Line& d, e_MergeDetails& mergeDetails, bool& bConflict, bool& bLineRemoved, int& src, bool bTwoInputs );
-
-enum e_MergeSrcSelector
-{
-   A=1,
-   B=2,
-   C=3
-};
-
+class RLPainter;
+class KActionCollection;
+class KToggleAction;
 class MergeResultWindow : public QWidget
 {
    Q_OBJECT
 public:
-   MergeResultWindow(
-      QWidget* pParent,
-      Options* pOptions,
-      QStatusBar* pStatusBar
-      );
+   MergeResultWindow(QWidget* pParent, Options* pOptions, QStatusBar* pStatusBar);
 
    void init(
-      const LineData* pLineDataA, int sizeA,
-      const LineData* pLineDataB, int sizeB,
-      const LineData* pLineDataC, int sizeC,
+      const LineData* pLineDataA, LineRef sizeA,
+      const LineData* pLineDataB, LineRef sizeB,
+      const LineData* pLineDataC, LineRef sizeC,
       const Diff3LineList* pDiff3LineList,
       TotalDiffStatus* pTotalDiffStatus
       );
-
+   void initActions(KActionCollection* ac);
    void reset();
 
    bool saveDocument( const QString& fileName, QTextCodec* pEncoding, e_LineEndStyle eLineEndStyle );
-   int getNrOfUnsolvedConflicts(int* pNrOfWhiteSpaceConflicts=0);
-   void choose(int selector);
-   void chooseGlobal(int selector, bool bConflictsOnly, bool bWhiteSpaceOnly );
+   int getNrOfUnsolvedConflicts(int* pNrOfWhiteSpaceConflicts=nullptr);
+   void choose(e_SrcSelector selector);
+   void chooseGlobal(e_SrcSelector selector, bool bConflictsOnly, bool bWhiteSpaceOnly );
 
    int getMaxTextWidth();     // width of longest text line
    int getNofLines();
@@ -138,7 +66,10 @@ public:
    void setSelection( int firstLine, int startPos, int lastLine, int endPos );
    void setOverviewMode( Overview::e_OverviewMode eOverviewMode );
    Overview::e_OverviewMode getOverviewMode();
-public slots:
+
+   void slotUpdateAvailabilities(const bool bMergeEditorVisible, const bool m_bTripleDiff);
+
+public Q_SLOTS:
    void setFirstLine(int firstLine);
    void setHorizScrollOffset(int horizScrollOffset);
 
@@ -162,8 +93,18 @@ public slots:
    void updateSourceMask();
    void slotStatusMessageChanged( const QString& );
 
-signals:
-   void scroll( int deltaX, int deltaY );
+   void slotChooseAEverywhere() { chooseGlobal(A, false, false); }
+   void slotChooseBEverywhere() { chooseGlobal(B, false, false); }
+   void slotChooseCEverywhere() { chooseGlobal(C, false, false); }
+   void slotChooseAForUnsolvedConflicts() { chooseGlobal(A, true, false); }
+   void slotChooseBForUnsolvedConflicts() { chooseGlobal(B, true, false); }
+   void slotChooseCForUnsolvedConflicts() { chooseGlobal(C, true, false); }
+   void slotChooseAForUnsolvedWhiteSpaceConflicts() { chooseGlobal(A, true, true); }
+   void slotChooseBForUnsolvedWhiteSpaceConflicts() { chooseGlobal(B, true, true); }
+   void slotChooseCForUnsolvedWhiteSpaceConflicts() { chooseGlobal(C, true, true); }
+
+ Q_SIGNALS:
+   void scrollMergeResultWindow( int deltaX, int deltaY );
    void modifiedChanged(bool bModified);
    void setFastSelectorRange( int line1, int nofLines );
    void sourceMask( int srcMask, int enabledMask );
@@ -175,22 +116,31 @@ signals:
    void noRelevantChangesDetected();
 
 private:
-   void merge(bool bAutoSolve, int defaultSelector, bool bConflictsOnly=false, bool bWhiteSpaceOnly=false );
+   void merge(bool bAutoSolve, e_SrcSelector defaultSelector, bool bConflictsOnly=false, bool bWhiteSpaceOnly=false );
    QString getString( int lineIdx );
 
-   Options* m_pOptions;
+   QAction* chooseAEverywhere = nullptr;
+   QAction* chooseBEverywhere = nullptr;
+   QAction* chooseCEverywhere = nullptr;
+   QAction* chooseAForUnsolvedConflicts = nullptr;
+   QAction* chooseBForUnsolvedConflicts = nullptr;
+   QAction* chooseCForUnsolvedConflicts = nullptr;
+   QAction* chooseAForUnsolvedWhiteSpaceConflicts = nullptr;
+   QAction* chooseBForUnsolvedWhiteSpaceConflicts = nullptr;
+   QAction* chooseCForUnsolvedWhiteSpaceConflicts = nullptr;
+
+   Options* m_pOptions = nullptr;
 
    const LineData* m_pldA;
    const LineData* m_pldB;
    const LineData* m_pldC;
-   int m_sizeA;
-   int m_sizeB;
-   int m_sizeC;
+   LineRef m_sizeA;
+   LineRef m_sizeB;
+   LineRef m_sizeC;
 
    const Diff3LineList* m_pDiff3LineList;
    TotalDiffStatus* m_pTotalDiffStatus;
 
-   bool m_bPaintingAllowed;
    int m_delayedDrawTimer;
    Overview::e_OverviewMode m_eOverviewMode;
    QString m_persistentStatusMessage;
@@ -200,25 +150,25 @@ private:
    class MergeEditLine
    {
    public:
-      MergeEditLine(Diff3LineList::const_iterator i, int src=0){m_id3l=i; m_src=src; m_bLineRemoved=false; }
-      void setConflict() { m_src=0; m_bLineRemoved=false; m_str=QString(); }
-      bool isConflict()  { return  m_src==0 && !m_bLineRemoved && m_str.isNull(); }
-      void setRemoved(int src=0)  { m_src=src; m_bLineRemoved=true; m_str=QString(); }
+      explicit MergeEditLine(const Diff3LineList::const_iterator &i, e_SrcSelector src=None){m_id3l=i; m_src=src; m_bLineRemoved=false; }
+      void setConflict() { m_src=None; m_bLineRemoved=false; m_str=QString(); }
+      bool isConflict()  { return  m_src==None && !m_bLineRemoved && m_str.isEmpty(); }
+      void setRemoved(e_SrcSelector src=None)  { m_src=src; m_bLineRemoved=true; m_str=QString(); }
       bool isRemoved()   { return m_bLineRemoved; }
       bool isEditableText() { return !isConflict() && !isRemoved(); }
-      void setString( const QString& s ){ m_str=s; m_bLineRemoved=false; m_src=0; }
+      void setString( const QString& s ){ m_str=s; m_bLineRemoved=false; m_src=None; }
       QString getString( const MergeResultWindow* );
-      bool isModified() { return ! m_str.isNull() ||  (m_bLineRemoved && m_src==0); }
+      bool isModified() { return ! m_str.isEmpty() ||  (m_bLineRemoved && m_src==None); }
 
-      void setSource( int src, bool bLineRemoved ) { m_src=src; m_bLineRemoved =bLineRemoved; }
-      int src() { return m_src; }
+      void setSource( e_SrcSelector src, bool bLineRemoved ) { m_src=src; m_bLineRemoved =bLineRemoved; }
+      e_SrcSelector src() { return m_src; }
       Diff3LineList::const_iterator id3l(){return m_id3l;}
       // getString() is implemented as MergeResultWindow::getString()
    private:
-      Diff3LineList::const_iterator m_id3l;
-      int m_src;         // 1, 2 or 3 for A, B or C respectively, or 0 when line is from neither source.
-      QString m_str;    // String when modified by user or null-string when orig data is used.
-      bool m_bLineRemoved;
+     Diff3LineList::const_iterator m_id3l;
+     e_SrcSelector m_src; // 1, 2 or 3 for A, B or C respectively, or 0 when line is from neither source.
+     QString m_str;       // String when modified by user or null-string when orig data is used.
+     bool m_bLineRemoved;
    };
 
    class MergeEditLineList : private std::list<MergeEditLine>
@@ -231,7 +181,7 @@ private:
       typedef std::list<MergeEditLine>::iterator iterator;
       typedef std::list<MergeEditLine>::reverse_iterator reverse_iterator;
       typedef std::list<MergeEditLine>::const_iterator const_iterator;
-      MergeEditLineList(){m_size=0; m_pTotalSize=0; }
+      MergeEditLineList(){m_size=0; m_pTotalSize=nullptr; }
       void clear()                             { ds(-m_size); BASE::clear();          }
       void push_back( const MergeEditLine& m)  { ds(+1); BASE::push_back(m);     }
       void push_front( const MergeEditLine& m) { ds(+1); BASE::push_front(m);    }
@@ -249,8 +199,8 @@ private:
       void splice(iterator destPos, MergeEditLineList& srcList, iterator srcFirst, iterator srcLast)
       {
          int* pTotalSize = getTotalSizePtr() ? getTotalSizePtr() : srcList.getTotalSizePtr();
-         srcList.setTotalSizePtr(0); // Force size-recalc after splice, because splice doesn't handle size-tracking
-         setTotalSizePtr(0);
+         srcList.setTotalSizePtr(nullptr); // Force size-recalc after splice, because splice doesn't handle size-tracking
+         setTotalSizePtr(nullptr);
          BASE::splice( destPos, srcList, srcFirst, srcLast );
          srcList.setTotalSizePtr( pTotalSize );
          setTotalSizePtr( pTotalSize );
@@ -258,8 +208,8 @@ private:
 
       void setTotalSizePtr(int* pTotalSize)
       {
-         if ( pTotalSize==0 && m_pTotalSize!=0 ) { *m_pTotalSize -= size(); }
-         else if ( pTotalSize!=0 && m_pTotalSize==0 ) { *pTotalSize += size(); }
+         if ( pTotalSize==nullptr && m_pTotalSize!=nullptr ) { *m_pTotalSize -= size(); }
+         else if ( pTotalSize!=nullptr && m_pTotalSize==nullptr ) { *pTotalSize += size(); }
          m_pTotalSize = pTotalSize;
       }
       int* getTotalSizePtr()
@@ -268,72 +218,68 @@ private:
       }
 
    private:
-      void ds(int deltaSize) 
+      void ds(int deltaSize)
       {
-         m_size+=deltaSize; 
-         if (m_pTotalSize!=0)  *m_pTotalSize+=deltaSize;
+         m_size+=deltaSize;
+         if (m_pTotalSize!=nullptr)  *m_pTotalSize+=deltaSize;
       }
    };
 
    friend class MergeEditLine;
 
-   struct MergeLine
+   class MergeLine
    {
-      MergeLine()
-      {
-         srcSelect=0; mergeDetails=eDefault; d3lLineIdx = -1; srcRangeLength=0;
-         bConflict=false; bDelta=false; bWhiteSpaceConflict=false;
-      }
-      Diff3LineList::const_iterator id3l;
-      int d3lLineIdx;  // Needed to show the correct window pos.
-      int srcRangeLength; // how many src-lines have this properties
-      e_MergeDetails mergeDetails;
-      bool bConflict;
-      bool bWhiteSpaceConflict;
-      bool bDelta;
-      int srcSelect;
-      MergeEditLineList mergeEditLineList;
-      void split( MergeLine& ml2, int d3lLineIdx2 ) // The caller must insert the ml2 after this ml in the m_mergeLineList
-      {
-         if ( d3lLineIdx2<d3lLineIdx || d3lLineIdx2 >= d3lLineIdx + srcRangeLength ) 
-            return; //Error
-         ml2.mergeDetails = mergeDetails;
-         ml2.bConflict = bConflict;
-         ml2.bWhiteSpaceConflict = bWhiteSpaceConflict;
-         ml2.bDelta = bDelta;
-         ml2.srcSelect = srcSelect;
-
-         ml2.d3lLineIdx = d3lLineIdx2;
-         ml2.srcRangeLength = srcRangeLength - (d3lLineIdx2-d3lLineIdx);
-         srcRangeLength = d3lLineIdx2-d3lLineIdx; // current MergeLine controls fewer lines
-         ml2.id3l = id3l;
-         for(int i=0; i<srcRangeLength; ++i)
-            ++ml2.id3l;
-
-         ml2.mergeEditLineList.clear();
-         // Search for best place to splice
-         for(MergeEditLineList::iterator i=mergeEditLineList.begin(); i!=mergeEditLineList.end();++i)
+      public:
+         Diff3LineList::const_iterator id3l;
+         int d3lLineIdx = -1;  // Needed to show the correct window pos.
+         int srcRangeLength = 0; // how many src-lines have this properties
+         e_MergeDetails mergeDetails = eDefault;
+         bool bConflict = false;
+         bool bWhiteSpaceConflict = false;
+         bool bDelta = false;
+         e_SrcSelector srcSelect = None;
+         MergeEditLineList mergeEditLineList;
+         void split( MergeLine& ml2, int d3lLineIdx2 ) // The caller must insert the ml2 after this ml in the m_mergeLineList
          {
-            if (i->id3l()==ml2.id3l)
+            if ( d3lLineIdx2<d3lLineIdx || d3lLineIdx2 >= d3lLineIdx + srcRangeLength )
+               return; //Error
+            ml2.mergeDetails = mergeDetails;
+            ml2.bConflict = bConflict;
+            ml2.bWhiteSpaceConflict = bWhiteSpaceConflict;
+            ml2.bDelta = bDelta;
+            ml2.srcSelect = srcSelect;
+
+            ml2.d3lLineIdx = d3lLineIdx2;
+            ml2.srcRangeLength = srcRangeLength - (d3lLineIdx2-d3lLineIdx);
+            srcRangeLength = d3lLineIdx2-d3lLineIdx; // current MergeLine controls fewer lines
+            ml2.id3l = id3l;
+            for(int i=0; i<srcRangeLength; ++i)
+               ++ml2.id3l;
+
+            ml2.mergeEditLineList.clear();
+            // Search for best place to splice
+            for(MergeEditLineList::iterator i=mergeEditLineList.begin(); i!=mergeEditLineList.end();++i)
             {
-               ml2.mergeEditLineList.splice( ml2.mergeEditLineList.begin(), mergeEditLineList, i, mergeEditLineList.end() );
-               return;
+               if (i->id3l()==ml2.id3l)
+               {
+                  ml2.mergeEditLineList.splice( ml2.mergeEditLineList.begin(), mergeEditLineList, i, mergeEditLineList.end() );
+                  return;
+               }
             }
+            ml2.mergeEditLineList.setTotalSizePtr( mergeEditLineList.getTotalSizePtr() );
+            ml2.mergeEditLineList.push_back(MergeEditLine(ml2.id3l));
          }
-         ml2.mergeEditLineList.setTotalSizePtr( mergeEditLineList.getTotalSizePtr() );
-         ml2.mergeEditLineList.push_back(MergeEditLine(ml2.id3l));
-      }
-      void join( MergeLine& ml2 ) // The caller must remove the ml2 from the m_mergeLineList after this call
-      {
-         srcRangeLength += ml2.srcRangeLength;
-         ml2.mergeEditLineList.clear();
-         mergeEditLineList.clear();
-         mergeEditLineList.push_back(MergeEditLine(id3l)); // Create a simple conflict
-         if ( ml2.bConflict ) bConflict = true;
-         if ( !ml2.bWhiteSpaceConflict ) bWhiteSpaceConflict = false;
-         if ( ml2.bDelta ) bDelta = true;
-      }
-   };
+         void join( MergeLine& ml2 ) // The caller must remove the ml2 from the m_mergeLineList after this call
+         {
+            srcRangeLength += ml2.srcRangeLength;
+            ml2.mergeEditLineList.clear();
+            mergeEditLineList.clear();
+            mergeEditLineList.push_back(MergeEditLine(id3l)); // Create a simple conflict
+            if ( ml2.bConflict ) bConflict = true;
+            if ( !ml2.bWhiteSpaceConflict ) bWhiteSpaceConflict = false;
+            if ( ml2.bDelta ) bDelta = true;
+         }
+      };
 
 private:
    static bool sameKindCheck( const MergeLine& ml1, const MergeLine& ml2 );
@@ -346,7 +292,7 @@ private:
       bool staysInPlace( bool bThreeInputs, Diff3LineList::const_iterator& iHistoryEnd );
    };
    typedef std::map<QString,HistoryMapEntry> HistoryMap;
-   void collectHistoryInformation( int src, Diff3LineList::const_iterator iHistoryBegin, Diff3LineList::const_iterator iHistoryEnd, HistoryMap& historyMap, std::list< HistoryMap::iterator >& hitList );
+   void collectHistoryInformation( e_SrcSelector src, Diff3LineList::const_iterator &iHistoryBegin, Diff3LineList::const_iterator &iHistoryEnd, HistoryMap& historyMap, std::list< HistoryMap::iterator >& hitList );
 
    typedef std::list<MergeLine> MergeLineList;
    MergeLineList m_mergeLineList;
@@ -370,27 +316,27 @@ private:
       );
    MergeLineList::iterator splitAtDiff3LineIdx( int d3lLineIdx );
 
-   virtual void paintEvent( QPaintEvent* e );
+   void paintEvent( QPaintEvent* e ) override;
 
    int getTextXOffset();
    QVector<QTextLayout::FormatRange> getTextLayoutForLine(int line, const QString& s, QTextLayout& textLayout );
    void myUpdate(int afterMilliSecs);
-   virtual void timerEvent(QTimerEvent*);
+   void timerEvent(QTimerEvent*) override;
    void writeLine(
-      MyPainter& p, int line, const QString& str,
+      RLPainter& p, int line, const QString& str,
       int srcSelect, e_MergeDetails mergeDetails, int rangeMark, bool bUserModified, bool bLineRemoved, bool bWhiteSpaceConflict
       );
    void setFastSelector(MergeLineList::iterator i);
    int convertToLine( int y );
-   bool event(QEvent*);
-   virtual void mousePressEvent ( QMouseEvent* e );
-   virtual void mouseDoubleClickEvent ( QMouseEvent* e );
-   virtual void mouseReleaseEvent ( QMouseEvent * );
-   virtual void mouseMoveEvent ( QMouseEvent * );
-   virtual void resizeEvent( QResizeEvent* e );
-   virtual void keyPressEvent( QKeyEvent* e );
-   virtual void wheelEvent( QWheelEvent* e );
-   virtual void focusInEvent( QFocusEvent* e );
+   bool event(QEvent*) override;
+   void mousePressEvent ( QMouseEvent* e ) override;
+   void mouseDoubleClickEvent ( QMouseEvent* e ) override;
+   void mouseReleaseEvent ( QMouseEvent * ) override;
+   void mouseMoveEvent ( QMouseEvent * ) override;
+   void resizeEvent( QResizeEvent* e ) override;
+   void keyPressEvent( QKeyEvent* e ) override;
+   void wheelEvent( QWheelEvent* e ) override;
+   void focusInEvent( QFocusEvent* e ) override;
 
    QPixmap m_pixmap;
    int m_firstLine;
@@ -419,10 +365,10 @@ private:
    bool deleteSelection2( QString& str, int& x, int& y,
                     MergeLineList::iterator& mlIt, MergeEditLineList::iterator& melIt );
    bool doRelevantChangesExist();
-public slots:
+public Q_SLOTS:
    void deleteSelection();
    void pasteClipboard(bool bFromSelection);
-private slots:
+private Q_SLOTS:
    void slotCursorUpdate();
 };
 
@@ -430,12 +376,13 @@ class QLineEdit;
 class QTextCodec;
 class QComboBox;
 class QLabel;
+
 class WindowTitleWidget : public QWidget
 {
    Q_OBJECT
 private:
    QLabel*      m_pLabel;
-   QLineEdit*   m_pFileNameLineEdit;
+   FileNameLineEdit*   m_pFileNameLineEdit;
    //QPushButton* m_pBrowseButton;
    QLabel*      m_pModifiedLabel;
    QLabel*      m_pLineEndStyleLabel;
@@ -444,21 +391,20 @@ private:
    QComboBox*   m_pEncodingSelector;
    Options*     m_pOptions;
 public:
-   WindowTitleWidget(Options* pOptions);
+   explicit WindowTitleWidget(Options* pOptions);
    QTextCodec* getEncoding();
    void        setFileName(const QString& fileName );
    QString     getFileName();
    void setEncodings( QTextCodec* pCodecForA, QTextCodec* pCodecForB, QTextCodec* pCodecForC );
-   void setEncoding( QTextCodec* pCodec );
+   void setEncoding( QTextCodec* pEncoding );
    void setLineEndStyles( e_LineEndStyle eLineEndStyleA, e_LineEndStyle eLineEndStyleB, e_LineEndStyle eLineEndStyleC);
    e_LineEndStyle getLineEndStyle();
 
-   bool eventFilter( QObject* o, QEvent* e );
-public slots:
+   bool eventFilter( QObject* o, QEvent* e ) override;
+public Q_SLOTS:
    void slotSetModified( bool bModified );
-//private slots:
+//private Q_SLOTS:
 //   void slotBrowseButtonClicked();
-
 };
 
 #endif
