@@ -14,40 +14,40 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
 #ifndef OPTION_DIALOG_H
 #define OPTION_DIALOG_H
 
-
 #include <QStringList>
 #include <QGroupBox>
 
 #include <KPageDialog>
 #include <KSharedConfig>
 
+#include <list>
+
 #include "options.h"
 
 class QLabel;
 class QPlainTextEdit;
 
-class OptionItem;
+class OptionItemBase;
 class OptionCheckBox;
 class OptionEncodingComboBox;
 class OptionLineEdit;
 class KKeyDialog;
 
-
 class OptionDialog : public KPageDialog
 {
    Q_OBJECT
 
 public:
 
-    OptionDialog( bool bShowDirMergeSettings, QWidget *parent = 0, char *name = 0 );
-    ~OptionDialog( void );
+    explicit OptionDialog( bool bShowDirMergeSettings, QWidget *parent = nullptr );
+    ~OptionDialog( void ) override;
     QString parseOptions( const QStringList& optionList );
     QString calcOptionHelp();
 
@@ -58,24 +58,25 @@ public:
 
     void setState(); // Must be called before calling exec();
 
-    void addOptionItem(OptionItem*);
+    void addOptionItem(OptionItemBase*);
     KKeyDialog* m_pKeyDialog;
-protected slots:
+protected Q_SLOTS:
     virtual void slotDefault( void );
     virtual void slotOk( void );
     virtual void slotApply( void );
+    //virtual void buttonClicked( QAbstractButton* );
+    virtual void helpRequested();
 
     void slotEncodingChanged();
     void slotHistoryMergeRegExpTester();
-    void slotIntegrateWithClearCase();
-    void slotRemoveClearCaseIntegration();
-signals:
+Q_SIGNALS:
     void applyDone();
 private:
     void resetToDefaults();
 
-    std::list<OptionItem*> m_optionItemList;
+    std::list<OptionItemBase*> m_optionItemList;
 
+    //QDialogButtonBox *mButtonBox;
     OptionCheckBox* m_pSameEncoding;
     OptionEncodingComboBox* m_pEncodingAComboBox;
     OptionCheckBox* m_pAutoDetectUnicodeA;
@@ -115,7 +116,7 @@ class FontChooser : public QGroupBox
    QPlainTextEdit* m_pExampleTextEdit;
    QLabel* m_pLabel;
 public:
-   FontChooser( QWidget* pParent );
+   explicit FontChooser( QWidget* pParent );
    QFont font();
    void setFont( const QFont&, bool );
 private slots:
